/* mouseclick.c --- simulate a mouse click.

   Copyright (C) 2005 Ralph Schleicher

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2,
   or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   As a special exception, Ralph Schleicher gives permission to link
   the code of this program with MATLAB from The Mathworks, Inc. (or
   with modified versions of MATLAB that use the same license as
   MATLAB), and distribute linked combinations including the two.
   You must obey the GNU General Public License in all respects for
   all of the code used other than with MATLAB.  If you modify this
   file, you may extend this exception to your version of the file,
   but you are not obligated to do so.  If you do not wish to do so,
   delete this exception statement from your version.  */

/* Commentary:

   On Unix, compile with

        mex mouseclick.c -lXtst -lXext -lX11

   On Windows (Windows 95, Windows NT 3.1, or newer), compile with

        mex mouseclick.c

   On Windows 98, Windows NT 4.0 Service Pack 3, or newer, you can
   compile with

        mex -DHAVE_SENDINPUT mouseclick.c  */


#undef USE_X11
#undef USE_WIN32

#ifdef WIN32
#define USE_WIN32 1
#else /* not WIN32 */
#define USE_X11 1
#endif /* not WIN32 */

#include <mex.h>

#if USE_X11
#include <X11/Xlib.h>
#include <X11/extensions/XTest.h>
#else /* not USE_X11 */
#if USE_WIN32
#include <windows.h>
#else /* not USE_WIN32 */
#error "Fix me"
#endif /* not USE_WIN32 */
#endif /* not USE_X11 */


void
mexFunction (int output_arg_count, mxArray *output_arg_vec[],
	     int input_arg_count, const mxArray *input_arg_vec[])
{
#if USE_X11

  Display *display;

  display = XOpenDisplay (NULL);
  if (display == NULL)
    mexErrMsgTxt ("Can not open display");

  XTestFakeButtonEvent (display, 1, True, 0);
  XTestFakeButtonEvent (display, 1, False, 1);

  XCloseDisplay (display);

#else /* not USE_X11 */
#if USE_WIN32
#if HAVE_SENDINPUT

  INPUT input;

  input.type = INPUT_MOUSE;
  input.mi.dx = 0;
  input.mi.dy = 0;
  input.mi.mouseData = 0;
  input.mi.time = 0;
  input.mi.dwExtraInfo = 0;

  input.mi.dwFlags = MOUSEEVENTF_LEFTDOWN;
  SendInput (1, &input, sizeof (INPUT));
  input.mi.dwFlags = MOUSEEVENTF_LEFTUP;
  SendInput (1, &input, sizeof (INPUT));

#else /* not HAVE_SENDINPUT */

  mouse_event (MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0);
  mouse_event (MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);

#endif /* not HAVE_SENDINPUT */
#else /* not USE_WIN32 */

  (void) 0;

#endif /* not USE_WIN32 */
#endif /* not USE_X11 */
}
